/**
 * Provide services and hold service configuration.
 * By default a service provider has two build in services
 * 'logger' and 'config'.
 *
 * @param {new(Object,InitializationContext) => serviceLoggerClass} serviceLoggerClass where the logging houtd go
 * @param {new(Object,InitializationContext) => serviceConfigClass} serviceConfigClass where the config comes from
 */
export function ServiceProviderMixin(superclass: any, serviceLoggerClass?: new (Object: any, InitializationContext: any) => new (Object: any, InitializationContext: any) => /*elided*/ any, serviceConfigClass?: new (Object: any, InitializationContext: any) => new (Object: any, InitializationContext: any) => /*elided*/ any): {
    new (config: any, ic?: any): {
        [x: string]: any;
        listeners: {};
        interceptorFactories: {};
        serviceFactories: {};
        services: {};
        ic: any;
        emit(name: any, ...args: any[]): void;
        addListener(name: any, listener: any): void;
        removeListener(name: any, listener: any): void;
        /**
         * By default be our own owner.
         * @return this
         */
        readonly owner: /*elided*/ any;
        /**
         * We are the service provider.
         * @return {boolean} true
         */
        readonly isServiceProvider: boolean;
        /**
         * Register service or interceptor factories.
         *
         * @param {[Function|string]} factories
         */
        registerFactories(factories: [Function | string]): Promise<void>;
        /**
         * Registers a interceptor factory for later use by
         * @see {instantiateInterceptor}.
         *
         * @param {new() => factory} factory
         * @returns {new() => factory} factory
         */
        registerInterceptorFactory(factory: new () => new () => /*elided*/ any): new () => new () => /*elided*/ any;
        unregisterInterceptorFactory(factory: any): void;
        /**
         * Instanciates an interceptor from its definition data.
         * @see {interceptorFactories}
         * @param {Object} definition data
         * @returns {Interceptor}
         */
        instantiateInterceptor(definition: any): Interceptor;
        serviceStateChanged(service: any, oldState: any, newState: any): void;
        registerServiceFactory(factory: any): Promise<any>;
        unregisterServiceFactory(factory: any): Promise<void>;
        registerService(service: any): Promise<any>;
        unregisterService(serviceName: any): Promise<void>;
        readonly serviceNames: string[];
        getService(name: any): any;
        declareService(config: any): Promise<any>;
        /**
         * Add a new service based on its configuration.
         * If a service for the name is already present and it has a matching type
         * then its configure() is called and returned.
         * Otherwise a new service will be created eventually replacing an already existing service with the same name.
         * @param {object} configs with
         *     name - the service name
         *     type - the service factory name - defaults to config.name
         * @return {Promise<Object>} resolving to the declared services
         */
        declareServices(configs: object): Promise<any>;
        /**
         * Start all registered services which hanving autostart set.
         */
        _start(): Promise<any[]>;
        /**
         * Stop all services.
         * @return {Promise} that resolves when all services are stopped
         */
        _stop(): Promise<any>;
    };
    [x: string]: any;
};
import { Interceptor } from "@kronos-integration/interceptor";
