declare const Service_base: new () => new () => /*elided*/ any;
/**
 * Service
 * The initial state is 'stopped'.
 * All services have at least three endpoints:
 * - log _out_: log events
 * - config _in_: configuration request
 * - command _in_: administrative actions to be executed by the step
 * @param {Object} config
 * @param {string} config.name
 * @param {string} config.logLevel
 * @param {boolean} config.autostart defaults to false
 * @param {string} config.description human readable description
 * @param {Object} config.endpoints will be merged with the build in ones
 * @param {InitializationContext} ic
 *
 * @property {Object} endpoints
 */
export class Service extends Service_base {
    static get description(): string;
    /**
     * Meta information for the config attributes.
     * - default optional default value of the attribute
     * - needsRestart optional modification requires a service restart
     * - setter(newValue,attribute) optional function to be used if simple value assignment is not enough
     * The Service class only defines the logLevel, and start/stop/restart timeout attribute
     * @return {Object}
     */
    static get configurationAttributes(): any;
    /**
     * Definition of the predefined endpoints.
     * - log _out_
     * - config _in_
     * @return {Object} predefined endpoints
     */
    static get endpoints(): any;
    constructor(config: any, ic: any);
    owner: any;
    logLevel: any;
    instantiateInterceptor(options: any): any;
    get configurationAttributes(): any;
    get type(): string;
    set description(desc: any);
    get description(): any;
    /**
     * Used in human readable state messages.
     * Besides the actual service name it may contain additional short hints.
     * @return {string}
     */
    get extendetName(): string;
    /**
     * Called when the service state changes.
     * Emits a serviceStateChanged event to the owner.
     * @param {string} oldState
     * @param {string} newState
     */
    stateChanged(origin: any, oldState: string, newState: string): void;
    stateTransitionRejection(rejected: any, newState: any): any;
    /**
     * Called when state transition is not allowed.
     * @param {string} action originating action name
     * @throws always
     */
    rejectWrongState(action: string): Promise<void>;
    /**
     * Deliver transition timeout.
     * @param {Object} transition
     * @return {number} milliseconds before throwing for a long running transition
     */
    timeoutForTransition(transition: any): number;
    /**
     * Opens all endpoint connections.
     */
    _start(): Promise<void>;
    /**
     * Closes all endpoint connections.
     */
    _stop(): Promise<void>;
    /**
     * Restart action.
     * default implementation does a _stop() and a _start()
     * @return {Promise} fulfills after start
     */
    _restart(): Promise<any>;
    /**
     * Restarts if in running mode.
     * Otherwise does nothing.
     * @returns {Promise} resolves when restart is done (or immediate if no restart triggered)
     */
    restartIfRunning(): Promise<any>;
    /**
     * Mapping of properties used in toString.
     * @return {Object}
     */
    get toStringAttributes(): any;
    /**
     * Base service is not a provider.
     * @return {boolean} false
     */
    get isServiceProvider(): boolean;
    toJSON(): any;
    /**
     * Deliver json representation.
     * @param {Object} options
     * @param {boolean} options.includeRuntimeInfo include runtime informtion like state
     * @param {boolean} options.includeDefaults include default endpoints
     * @param {boolean} options.includeName include name of the service
     * @param {boolean} options.includeConfig include config attributes
     * @param {boolean} options.includePrivate include private config attributes
     * @return {Object} json representation
     */
    toJSONWithOptions(options: {
        includeRuntimeInfo: boolean;
        includeDefaults: boolean;
        includeName: boolean;
        includeConfig: boolean;
        includePrivate: boolean;
    }): any;
    /**
     * Should we start when beeing registered.
     * @return {boolean} false
     */
    get autostart(): boolean;
    /**
     * Takes attribute values from config parameters
     * and copies them over to the object.
     * Copying is done according to configurationAttributes.
     * Which means we loop over all configuration attributes.
     * and then for each attribute decide if we use the default, call a setter function
     * or simply assign the attribute value.
     * @param {Object} config
     * @return {Set} of modified attributes
     */
    _configure(config: any): Set<any>;
    /**
     * Use new configuration.
     * Internally calls _configure(config) as the constructor does.
     * If attribute with needsRestart are touched the restartIfRunning method
     * will be called.
     * @param {Object} config
     * @return {Promise} fillfills when config is applied
     */
    configure(config: any): Promise<any>;
    /**
     * Adds service name to the log event.
     * @param {string} level the log level
     * @param {Object} arg log content
     */
    log(level: string, arg: any): void;
    #private;
}
export {};
